﻿using System.Collections.Generic;

namespace ProgramTree
{
    public enum AssignType { Assign, AssignPlus, AssignMinus, AssignMult, AssignDivide };

    public class Node // базовый класс для всех узлов    
    {
    }

    public class ExprNode : Node // базовый класс для всех выражений
    {
    }

    public class IdNode : ExprNode
    {
        public string Name { get; set; }
        public IdNode(string name) { Name = name; }
    }

    public class IntNumNode : ExprNode
    {
        public int Num { get; set; }
        public IntNumNode(int num) { Num = num; }
    }

    public class StatementNode : Node // базовый класс для всех операторов
    {
    }

    public class AssignNode : StatementNode
    {
        public IdNode Id { get; set; }
        public ExprNode Expr { get; set; }
        public AssignType AssOp { get; set; }
        public AssignNode(IdNode id, ExprNode expr, AssignType assop = AssignType.Assign)
        {
            Id = id;
            Expr = expr;
            AssOp = assop;
        }
    }

    public class CycleNode : StatementNode
    {
        public ExprNode Expr { get; set; }
        public StatementNode Stat { get; set; }
        public CycleNode(ExprNode expr, StatementNode stat)
        {
            Expr = expr;
            Stat = stat;
        }
    }
    
    public class WhileNode : StatementNode
    {
        public ExprNode Expr { get; set; }
        public StatementNode Stat { get; set; }
        public WhileNode(ExprNode expr, StatementNode stat)
        {
            Expr = expr;
            Stat = stat;
        }
    }
    
    public class RepeatNode : StatementNode
    {
        public ExprNode Expr { get; set; }
        public BlockNode Stlist { get; set; }
        public RepeatNode(BlockNode stlist, ExprNode expr)
        {
            Expr = expr;
            Stlist = stlist;
        }
    }
    
    public class ForNode : StatementNode
    {
        public StatementNode Assign { get; set; }
        public ExprNode Expr { get; set; }
        public StatementNode Stat { get; set; }
        public ForNode(StatementNode assign, ExprNode expr, StatementNode statement)
        {
            Expr = expr;
            Assign = assign;
            Stat = statement;
        }
    }
    
    public class WriteNode : StatementNode
    {
        public ExprNode Expr { get; set; }
        
        public WriteNode(ExprNode expr)
        {
            Expr = expr;
        }
    }
    
    public class IfNode : StatementNode
    {
        public ExprNode Expr { get; set; }
        public StatementNode IfStat { get; set; }
        public StatementNode ElseStat { get; set; }
        public IfNode(ExprNode expr, StatementNode ifStat, StatementNode elseStat)
        {
            Expr = expr;
            IfStat = ifStat;
            ElseStat = elseStat;
        }
        
        public IfNode(ExprNode expr, StatementNode ifStat)
        {
            Expr = expr;
            IfStat = ifStat;
            ElseStat = null;
        }
    }
    
    public class VarNode : StatementNode
    {
        public ExprListNode Exprs { get; set; }
        public VarNode(ExprListNode exprs)
        {
            Exprs = exprs;
        }
        
    }
    
    public class BinaryNode : ExprNode
    {
        public ExprNode ExprLeft { get; set; }
        public ExprNode ExprRight { get; set; }
        public char Operation { get; set; }
        public BinaryNode(ExprNode left, ExprNode right, char operation)
        {
            ExprLeft = left;
            ExprRight = right;
            Operation = operation;
        }
        
    }

    public class ExprListNode : StatementNode
    {
        public List<ExprNode> ExprList = new List<ExprNode>();
        public ExprListNode(ExprNode expr)
        {
            Add(expr);
        }
        public void Add(ExprNode expr)
        {
            ExprList.Add(expr);
        }
    }
    public class BlockNode : StatementNode
    {
        public List<StatementNode> StList = new List<StatementNode>();
        public BlockNode(StatementNode stat)
        {
            Add(stat);
        }
        public void Add(StatementNode stat)
        {
            StList.Add(stat);
        }
    }

}