﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ProgramTree;

namespace SimpleLang.Visitors
{
    public class ExprComplexityVisitor : AutoVisitor
    {
        private List<int> exprRanks = new List<int>();
        private bool isFirstLayer = true;
        // список должен содержать сложность каждого выражения, встреченного при обычном порядке обхода AST
        public List<int> getComplexityList()
        {
            return exprRanks;
        }

        private int getRank(char op)
        {
            switch (op)
            {
                case '*':
                case '/': return 3;
                case '+':
                case '-': return 1;
                default: throw new ArgumentException();
            }
        }

        public override void VisitIntNumNode(IntNumNode num)
        {
            if (isFirstLayer)
            {
                exprRanks.Add(0);
            }
        }

        public override void VisitBinOpNode(BinOpNode binop)
        {
            if (!isFirstLayer)
            {
                exprRanks[exprRanks.Count - 1] += getRank(binop.Op);
                binop.Left.Visit(this);
                binop.Right.Visit(this);
                return;
            }
            isFirstLayer = false;
            exprRanks.Add(getRank(binop.Op));
            binop.Left.Visit(this);
            binop.Right.Visit(this);
            isFirstLayer = true;
        }
    }
}
