using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ProgramTree;


namespace SimpleLang.Visitors
{
    public class MaxIfCycleNestVisitor : AutoVisitor
    {
        public int MaxNest = 0;
        private int currentNest = 0;
        private bool isFirstLayer = true;
        private bool isDeepening = false;

        public override void VisitCycleNode(CycleNode c)
        {
            if (!isFirstLayer)
            {
                if (isDeepening)
                {
                    currentNest++;
                    isDeepening = false;
                }
                c.Expr.Visit(this);
                isDeepening = true;
                c.Stat.Visit(this);
                isDeepening = false;
                return;
            }
            isFirstLayer = false;
            currentNest = 1;
            c.Expr.Visit(this);
            isDeepening = true;
            c.Stat.Visit(this);
            isFirstLayer = true;
            if (currentNest > MaxNest)
            {
                MaxNest = currentNest;
            }
        }

        public override void VisitIfNode(IfNode n)
        {
            if (!isFirstLayer)
            {
                if (isDeepening)
                {
                    currentNest++;
                    isDeepening = false;
                }
                n.Expr.Visit(this);
                isDeepening = true;
                n.IfStat.Visit(this);
                isDeepening = false;
                if (n.ElseStat != null)
                {
                    isDeepening = true;
                    n.ElseStat.Visit(this);
                    isDeepening = false;
                }
                return;
            }
            isFirstLayer = false;
            currentNest = 1;
            n.Expr.Visit(this);
            isDeepening = true;
            n.IfStat.Visit(this);
            isDeepening = false;
            if (n.ElseStat != null)
            {
                isDeepening = true;
                n.ElseStat.Visit(this);
                isDeepening = false;
            }
            isFirstLayer = true;
            if (currentNest > MaxNest)
            {
                MaxNest = currentNest;
            }
        }
    }
}