using System;
using NUnit.Framework;
using SimpleScanner;
using SimpleParser;
using System.Text;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace TestASTParser
{
    public class ASTParserTests
    {
        public static JObject Parse(string text)
        {
            Scanner scanner = new Scanner();
            scanner.SetSource(text, 0);

            Parser parser = new Parser(scanner);

            var b = parser.Parse();
            if (!b)
                Assert.Fail("программа не распознана");
            else
            {
                JsonSerializerSettings jsonSettings = new JsonSerializerSettings();
                jsonSettings.Formatting = Newtonsoft.Json.Formatting.Indented;
                jsonSettings.TypeNameHandling = TypeNameHandling.All;
                string output = JsonConvert.SerializeObject(parser.root, jsonSettings);
                return JObject.Parse(output);
            }

            return null;

        }
    }
    
    [TestFixture]
    public class WhileTests
    {
        
        [Test]
        public void TestWhile()
        {
            var tree = ASTParserTests.Parse("begin while 2 do a:=2 end");
            Assert.AreEqual("ProgramTree.WhileNode, SimpleLang", (string)tree["StList"]["$values"][0]["$type"]);   
            Assert.AreEqual("ProgramTree.IntNumNode, SimpleLang", (string)tree["StList"]["$values"][0]["Expr"]["$type"]);
            Assert.AreEqual("2", ((string)tree["StList"]["$values"][0]["Expr"]["Num"]).Trim());
            Assert.AreEqual("ProgramTree.AssignNode, SimpleLang", (string)tree["StList"]["$values"][0]["Stat"]["$type"]);
        }
    }
    
    [TestFixture]
    public class RepeatTests
    {
        
        [Test]
        public void TestRepeat()
        {
            var tree = ASTParserTests.Parse("begin repeat a:=2 until 2 end");
            Assert.AreEqual("ProgramTree.RepeatNode, SimpleLang", (string)tree["StList"]["$values"][0]["$type"]);
            // TODO: проверить узлы содержимого repeat
        }
    }
    
    [TestFixture]
    public class ForTests
    {
        
        [Test]
        public void TestFor()
        {
            var tree = ASTParserTests.Parse("begin for i:=2 to 10 do a:=2 end");
            Assert.AreEqual("ProgramTree.ForNode, SimpleLang", (string)tree["StList"]["$values"][0]["$type"]);
            // TODO: проверить узлы содержимого for
        }
    }
    
    [TestFixture]
    public class WriteTests
    {
        
        [Test]
        public void TestWrite()
        {
            var tree = ASTParserTests.Parse("begin write(2) end");
            Assert.AreEqual("ProgramTree.WriteNode, SimpleLang", (string)tree["StList"]["$values"][0]["$type"]);
            // TODO: проверить содержимое write
        }
    }
    
    [TestFixture]
    public class ExtraTests
    {
        
        [Test]
        public void TestIf()
        {
            var tree = ASTParserTests.Parse("begin if 1 then a:=12; if 5 then b:=666 else b:=777 end");
            Assert.AreEqual("ProgramTree.IfNode, SimpleLang", (string)tree["StList"]["$values"][0]["$type"]);
            Assert.AreEqual("ProgramTree.IntNumNode, SimpleLang", (string)tree["StList"]["$values"][0]["Expr"]["$type"]);
            Assert.AreEqual("1", ((string) tree["StList"]["$values"][0]["Expr"]["Num"]).Trim());
            Assert.AreEqual("ProgramTree.AssignNode, SimpleLang", (string)tree["StList"]["$values"][0]["IfStat"]["$type"]);
            Assert.AreEqual("12", ((string) tree["StList"]["$values"][0]["IfStat"]["Expr"]["Num"]).Trim());
            Assert.AreEqual(null, (string)tree["StList"]["$values"][0]["ElseStat"]);
            
            Assert.AreEqual("ProgramTree.IfNode, SimpleLang", (string)tree["StList"]["$values"][1]["$type"]);
            Assert.AreEqual("ProgramTree.IntNumNode, SimpleLang", (string)tree["StList"]["$values"][1]["Expr"]["$type"]);
            Assert.AreEqual("5", ((string) tree["StList"]["$values"][1]["Expr"]["Num"]).Trim());
            Assert.AreEqual("ProgramTree.AssignNode, SimpleLang", (string)tree["StList"]["$values"][1]["IfStat"]["$type"]);
            Assert.AreEqual("b", ((string) tree["StList"]["$values"][1]["IfStat"]["Id"]["Name"]).Trim());
            Assert.AreEqual("ProgramTree.AssignNode, SimpleLang", (string)tree["StList"]["$values"][1]["ElseStat"]["$type"]);
            Assert.AreEqual("777", ((string) tree["StList"]["$values"][1]["ElseStat"]["Expr"]["Num"]).Trim());
        }
        
        [Test]
        public void TestVarDef()
        {
            var tree = ASTParserTests.Parse("begin var a,b,c end");
            Assert.AreEqual("ProgramTree.VarNode, SimpleLang", (string)tree["StList"]["$values"][0]["$type"]);
            Assert.AreEqual("ProgramTree.ExprListNode, SimpleLang", (string)tree["StList"]["$values"][0]["Exprs"]["$type"]);
            Assert.AreEqual("ProgramTree.IdNode, SimpleLang", (string)tree["StList"]["$values"][0]["Exprs"]["ExprList"]["$values"][0]["$type"]);
            Assert.AreEqual("a", ((string) tree["StList"]["$values"][0]["Exprs"]["ExprList"]["$values"][0]["Name"]).Trim());
            Assert.AreEqual("ProgramTree.IdNode, SimpleLang", (string)tree["StList"]["$values"][0]["Exprs"]["ExprList"]["$values"][1]["$type"]);
            Assert.AreEqual("b", ((string) tree["StList"]["$values"][0]["Exprs"]["ExprList"]["$values"][1]["Name"]).Trim());
            Assert.AreEqual("ProgramTree.IdNode, SimpleLang", (string)tree["StList"]["$values"][0]["Exprs"]["ExprList"]["$values"][2]["$type"]);
            Assert.AreEqual("c", ((string) tree["StList"]["$values"][0]["Exprs"]["ExprList"]["$values"][2]["Name"]).Trim());
        }
        
        [Test]
        public void TestBinary()
        {
            var tree = ASTParserTests.Parse("begin x:=y*z+1/(1-y) end");
            Assert.AreEqual("ProgramTree.AssignNode, SimpleLang", (string)tree["StList"]["$values"][0]["$type"]);
            Assert.AreEqual("ProgramTree.BinaryNode, SimpleLang", (string)tree["StList"]["$values"][0]["Expr"]["$type"]);
            Assert.AreEqual("+", ((string) tree["StList"]["$values"][0]["Expr"]["Operation"]).Trim());
            Assert.AreEqual("ProgramTree.BinaryNode, SimpleLang", (string)tree["StList"]["$values"][0]["Expr"]["ExprLeft"]["$type"]);
            Assert.AreEqual("*", ((string) tree["StList"]["$values"][0]["Expr"]["ExprLeft"]["Operation"]).Trim());
            Assert.AreEqual("ProgramTree.IdNode, SimpleLang", (string)tree["StList"]["$values"][0]["Expr"]["ExprLeft"]["ExprLeft"]["$type"]);
            Assert.AreEqual("y", ((string) tree["StList"]["$values"][0]["Expr"]["ExprLeft"]["ExprLeft"]["Name"]).Trim());
            Assert.AreEqual("ProgramTree.IdNode, SimpleLang", (string)tree["StList"]["$values"][0]["Expr"]["ExprLeft"]["ExprRight"]["$type"]);
            Assert.AreEqual("z", ((string) tree["StList"]["$values"][0]["Expr"]["ExprLeft"]["ExprRight"]["Name"]).Trim());
            Assert.AreEqual("ProgramTree.BinaryNode, SimpleLang", (string)tree["StList"]["$values"][0]["Expr"]["ExprRight"]["$type"]);
            Assert.AreEqual("1", ((string) tree["StList"]["$values"][0]["Expr"]["ExprRight"]["ExprLeft"]["Num"]).Trim());
            Assert.AreEqual("-", ((string) tree["StList"]["$values"][0]["Expr"]["ExprRight"]["ExprRight"]["Operation"]).Trim());
            Assert.AreEqual("1", ((string) tree["StList"]["$values"][0]["Expr"]["ExprRight"]["ExprRight"]["ExprLeft"]["Num"]).Trim());
            Assert.AreEqual("y", ((string) tree["StList"]["$values"][0]["Expr"]["ExprRight"]["ExprRight"]["ExprRight"]["Name"]).Trim());
        }
    }
}